==> xpkg-v1.0.20.tar: xpkg-v1.0.20/gin/ctxUtils/ctx.go <==
package ctxUtils

import (
	"context"
	"errors"

	xsf "gitlab.wanzhuangkj.com/tush/xpkg/utils/xsf"

	"github.com/gin-gonic/gin"
	"github.com/spf13/cast"
	"go.uber.org/zap"
)

const (
	GinContextKey       = "gin_context"
	KeyConfigFile       = "config_file"
	KeyConfigCenter     = "config_center"
	HeaderXTimestampKey = "Timestamp"
	KeyReqBody          = "req-body"
	KeyRspBody          = "rsp-body"
	KeyTid              = "tid"
	KeyClientIP         = "client_ip"
	KeyUID              = "userId"
	KeyUType            = "userType"
	KeyCompanyID        = "companyId"
	KeyShopID           = "shopID"
	KeyUName            = "uname"
	KeyToken            = "token"
	KeyUser             = "user"
)

var (
	HeaderXRequestIDKey = "X-Request-ID"
	ContextTraceIDKey   = "X-Request-ID"
)

var (
	ErrorGinContextNotFound = errors.New("gin context not found")
)

// GetGinCtxTraceID get request id from gin.Context
func GetGinCtxTraceID(c *gin.Context) string {
	if v, isExist := c.Get(ContextTraceIDKey); isExist {
		if requestID, ok := v.(string); ok {
			return requestID
		}
	}
	return ""
}

// GinTraceIDField get request id field from gin.Context
func GinTraceIDField(c *gin.Context) zap.Field {
	return zap.String(ContextTraceIDKey, GetGinCtxTraceID(c))
}

// CtxRequestID get request id from context.Context
func CtxRequestID(c context.Context) string {
	v := c.Value(ContextTraceIDKey)
	if str, ok := v.(string); ok {
		return str
	}
	return ""
}

// CtxXRequestID get request id from context.Context
func CtxXRequestID(c context.Context) string {
	v := c.Value(HeaderXRequestIDKey)
	if str, ok := v.(string); ok {
		return str
	}
	return ""
}

func CtxGetGinCtx(c context.Context) *gin.Context {
	if str, ok := c.Value(GinContextKey).(*gin.Context); ok {
		return str
	}
	return nil
}

// CtxTraceIDField get request id field from context.Context
func CtxTraceIDField(c context.Context) zap.Field {
	return zap.String(ContextTraceIDKey, CtxRequestID(c))
}

func GetGinUserID(c *gin.Context) xsf.ID {
	return getGinVal[xsf.ID](c, KeyUID)
}

func GetGinShopID(c *gin.Context) xsf.ID {
	return getGinVal[xsf.ID](c, KeyShopID)
}

func GetCtxUserID(c context.Context) xsf.ID {
	return getCtxVal[xsf.ID](c, KeyUID)
}

func GetCtxShopID(c context.Context) xsf.ID {
	return getCtxVal[xsf.ID](c, KeyShopID)
}
func GetCtxString(c context.Context, key string) string {
	val := c.Value(key)
	if val != nil {
		return cast.ToString(val)
	}
	return ""
}

func GetCtxUserToken(c context.Context) string {
	token := c.Value(KeyToken)
	if token != nil {
		return cast.ToString(token)
	}
	return ""
}

func GetGinCtxTid(c *gin.Context) string {
	tid := c.GetString(KeyTid)
	if tid == "" {
		tid = GenerateTid()
		c.Set(KeyTid, tid)
	}
	return tid
}

func GenerateTid() string {
	return xsf.Generate().String()
}

func GetCtxTid(ctx context.Context) string {
	tid := ""
	tidVal := ctx.Value(KeyTid)
	if tidVal != nil {
		if str, ok := tidVal.(string); ok {
			tid = str
		}
	}
	if tid == "" {
		tid = GenerateTid()
	}
	return tid
}

// GetClientIP get ip from context.Context, the client ip is set in gin.Context
func GetClientIP(ctx context.Context) string {
	ip := ""
	ipVal := ctx.Value(KeyClientIP)
	if ipVal != nil {
		if str, ok := ipVal.(string); ok {
			ip = str
		}
	}
	return ip
}

func WrapCtx(c *gin.Context) context.Context {
	ctx := context.WithValue(c.Request.Context(), ContextTraceIDKey, c.GetString(ContextTraceIDKey)) //nolint
	for k, v := range c.Keys {
		ctx = context.WithValue(ctx, k, v) //nolint
	}
	ctx = context.WithValue(ctx, GinContextKey, c) //nolint
	return ctx
}

func ToGinCtx(ctx context.Context) (*gin.Context, error) {
	if c, ok := ctx.Value(GinContextKey).(*gin.Context); ok {
		return c, nil
	}
	return nil, ErrorGinContextNotFound
}

func GetGinCtx(ctx context.Context) (*gin.Context, error) {
	if c, ok := ctx.Value(GinContextKey).(*gin.Context); ok {
		return c, nil
	}
	return nil, ErrorGinContextNotFound
}

func GetCtxUID(c context.Context) uint {
	return getCtxVal[uint](c, KeyUID)
}

func GetCtxUType(c context.Context) uint {
	return getCtxVal[uint](c, KeyUType)
}

func GetCtxCompanyID(c context.Context) uint {
	return getCtxVal[uint](c, KeyCompanyID)
}

func getGinVal[T any](c *gin.Context, key string) T {
	val, exists := c.Get(key)
	if exists {
		if ret, ok := val.(T); ok {
			return ret
		}
	}
	var t T
	return t
}

func getCtxVal[T any](c context.Context, key string) T {
	val := c.Value(key)
	if val != nil {
		if ret, ok := val.(T); ok {
			return ret
		}
	}
	var t T
	return t
}

func GetGinReq(c *gin.Context) []byte {
	v, ok1 := c.Get(KeyReqBody)
	if ok1 {
		ret, ok2 := v.([]byte)
		if ok2 {
			return ret
		}
	}
	return nil
}

func GetGinRsp(c *gin.Context) []byte {
	v, ok1 := c.Get(KeyRspBody)
	if ok1 {
		ret, ok2 := v.([]byte)
		if ok2 {
			return ret
		}
	}
	return nil
}

func SetVal(ctx context.Context, key string, val interface{}) context.Context {
	if ginCtx, err := GetGinCtx(ctx); err == nil && ginCtx != nil {
		ginCtx.Set(key, val)
	}
	return context.WithValue(ctx, key, val)
}

==> xpkg-v1.0.36.tar: xpkg-v1.0.36/gin/ctxUtils/ctx.go <==
package ctxUtils

import (
	"context"
	"errors"
	"fmt"
	"strings"
	"time"

	xsf "gitlab.wanzhuangkj.com/tush/xpkg/utils/xsf"

	"github.com/gin-gonic/gin"
	"github.com/spf13/cast"
	"go.uber.org/zap"
)

const (
	GinContextKey       = "ginContext"
	HeaderXTimestampKey = "Timestamp"
	KeyReqBody          = "reqBody"
	KeyRspBody          = "rspBody"

	KeyOApiReqBody = "oApiReqBody%s"
	KeyOApiRspBody = "oApiRspBody%s"

	KeyAppName      = "appName"
	KeyApiStartTime = "apiStartTime"
	KeyClientIP     = "clientIP"
	KeyUID          = "userId"
	KeyUType        = "userType"
	KeyCompanyID    = "companyId"
	KeyShopID       = "shopID"
	KeyUName        = "uname"
	KeyToken        = "token"
	KeyCost         = "X-Request-Cost"
)

var (
	HeaderXRequestIDKey = "X-Request-ID"
	ContextTraceIDKey   = "X-Request-ID"
)

var (
	ErrorGinContextNotFound = errors.New("gin context not found")
)

// GetGinCtxTraceID get request id from gin.Context
func GetGinCtxTraceID(c *gin.Context) string {
	if v, isExist := c.Get(ContextTraceIDKey); isExist {
		if requestID, ok := v.(string); ok {
			return requestID
		}
	}
	return ""
}

// GinTraceIDField get request id field from gin.Context
func GinTraceIDField(c *gin.Context) zap.Field {
	return zap.String(ContextTraceIDKey, GetGinCtxTraceID(c))
}

// CtxRequestID get request id from context.Context
func CtxRequestID(c context.Context) string {
	v := c.Value(ContextTraceIDKey)
	if str, ok := v.(string); ok {
		return str
	}
	return ""
}

// CtxXRequestID get request id from context.Context
func CtxXRequestID(c context.Context) string {
	v := c.Value(HeaderXRequestIDKey)
	if str, ok := v.(string); ok {
		return str
	}
	return ""
}

func CtxGetGinCtx(c context.Context) *gin.Context {
	if str, ok := c.Value(GinContextKey).(*gin.Context); ok {
		return str
	}
	return nil
}

// CtxTraceIDField get request id field from context.Context
func CtxTraceIDField(c context.Context) zap.Field {
	return zap.String(ContextTraceIDKey, CtxRequestID(c))
}

func GetGinUserID(c *gin.Context) xsf.ID {
	return getGinVal[xsf.ID](c, KeyUID)
}

func GetGinShopID(c *gin.Context) xsf.ID {
	return getGinVal[xsf.ID](c, KeyShopID)
}

func GetCtxUserID(c context.Context) xsf.ID {
	return getCtxVal[xsf.ID](c, KeyUID)
}

func GetCtxShopID(c context.Context) xsf.ID {
	return getCtxVal[xsf.ID](c, KeyShopID)
}
func GetCtxString(c context.Context, key string) string {
	val := c.Value(key)
	if val != nil {
		return cast.ToString(val)
	}
	return ""
}

func GetCtxUserToken(c context.Context) string {
	token := c.Value(KeyToken)
	if token != nil {
		return cast.ToString(token)
	}
	return ""
}

func GetGinCtxTid(c *gin.Context) string {
	tid := c.GetString(ContextTraceIDKey)
	if tid == "" {
		tid = GenerateTid()
		c.Set(ContextTraceIDKey, tid)
	}
	return tid
}

func GenerateTid() string {
	return xsf.Generate().String()
}

func GetCtxTid(ctx context.Context) string {
	tid := ""
	tidVal := ctx.Value(ContextTraceIDKey)
	if tidVal != nil {
		if str, ok := tidVal.(string); ok {
			tid = str
		}
	}
	if tid == "" {
		tid = GenerateTid()
	}
	return tid
}

// GetClientIP get ip from context.Context, the client ip is set in gin.Context
func GetClientIP(ctx context.Context) string {
	ip := ""
	ipVal := ctx.Value(KeyClientIP)
	if ipVal != nil {
		if str, ok := ipVal.(string); ok {
			ip = str
		}
	}
	return ip
}

func WrapCtx(c *gin.Context) context.Context {
	ctx := context.WithValue(c.Request.Context(), ContextTraceIDKey, c.GetString(ContextTraceIDKey)) //nolint
	for k, v := range c.Keys {
		ctx = context.WithValue(ctx, k, v) //nolint
	}
	ctx = context.WithValue(ctx, GinContextKey, c) //nolint
	return ctx
}

func ToGinCtx(ctx context.Context) (*gin.Context, error) {
	if c, ok := ctx.Value(GinContextKey).(*gin.Context); ok {
		return c, nil
	}
	return nil, ErrorGinContextNotFound
}

func GetGinCtx(ctx context.Context) (*gin.Context, error) {
	if c, ok := ctx.Value(GinContextKey).(*gin.Context); ok {
		return c, nil
	}
	return nil, ErrorGinContextNotFound
}

func GetCtxUID(c context.Context) uint {
	return getCtxVal[uint](c, KeyUID)
}

func GetCtxUType(c context.Context) uint {
	return getCtxVal[uint](c, KeyUType)
}

func GetCtxCompanyID(c context.Context) uint {
	return getCtxVal[uint](c, KeyCompanyID)
}

func getGinVal[T any](c *gin.Context, key string) T {
	val, exists := c.Get(key)
	if exists {
		if ret, ok := val.(T); ok {
			return ret
		}
	}
	var t T
	return t
}

func getCtxVal[T any](c context.Context, key string) T {
	val := c.Value(key)
	if val != nil {
		if ret, ok := val.(T); ok {
			return ret
		}
	}
	var t T
	return t
}

func GetGinReq(c *gin.Context) []byte {
	v, ok1 := c.Get(KeyReqBody)
	if ok1 {
		ret, ok2 := v.([]byte)
		if ok2 {
			return ret
		}
	}
	return nil
}

func GetGinRsp(c *gin.Context) []byte {
	v, ok1 := c.Get(KeyRspBody)
	if ok1 {
		ret, ok2 := v.([]byte)
		if ok2 {
			return ret
		}
	}
	return nil
}

func SetVal(ctx context.Context, key string, val interface{}) context.Context {
	if ginCtx, err := GetGinCtx(ctx); err == nil && ginCtx != nil {
		ginCtx.Set(key, val)
	}
	return context.WithValue(ctx, key, val)
}

func AddApiCost(c *gin.Context, appName string, start time.Time) {
	cost := c.Writer.Header().Get(KeyCost)
	currentCost := fmt.Sprintf("%s:%d", appName, time.Since(start).Milliseconds())
	if cost == "" {
		cost = currentCost
	} else {
		cost = fmt.Sprintf("%s,%s", currentCost, cost)
	}
	c.Writer.Header().Set(KeyCost, cost)
}

func SetApiCost(ctx context.Context, header map[string][]string) {
	if ginCtx, _ := GetGinCtx(ctx); ginCtx != nil && len(header) > 0 {
		cost := ""
		for k, v := range header {
			if k == KeyCost {
				cost = strings.Join(v, ",")
			}
		}
		if cost != "" {
			ginCtx.Writer.Header().Set(KeyCost, cost)
		}
	}
}
